# -*- coding: UTF-8 -*-
#
#  FstabTest.py : Unittest for Fstab.py
#  Copyright (C) 2007 Mertens Florent <flomertens@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

TESTDIR = "/tmp/test/"
TESTFSTAB = TESTDIR + "fstab"


import os
import sys
import shutil
import random
from Fstab import *
from FstabUtility import *
from DiskInfo import *
import FstabData
import unittest


if os.path.exists(TESTDIR) :
    shutil.rmtree(TESTDIR)
os.mkdir(TESTDIR)
info = get_diskinfo_backend()()
good_dev = []
extra_good_dev = []
extra_extra_good_dev = []
for device in info.list(ignored = False) :
    good_dev.append(device)
    dev = info[device]
    if dev.has_key("FS_UUID") and len(info.search(dev["FS_UUID"], keys = ["FS_UUID"])) < 2 :
        if os.path.islink("/dev/disk/by-uuid/%s" % dev["FS_UUID"]) :
            extra_good_dev.append(device)
        if dev.has_key("FS_LABEL") and len(info.search(dev["FS_LABEL"], keys = ["FS_LABEL"])) < 2 :
            if os.path.islink("/dev/disk/by-label/%s" % dev["FS_LABEL"]) :
                extra_extra_good_dev.append(device)

bad_dev = [ k["DEVICE"] for k in info if k["IGNORE"] ]
good_comment = "  \t  #0    1 2         3\t4   5        6  \t \n"

def get_path(device):
    
    return TESTDIR + device
 
def create_fstab() :

    fstab = ""
    for dev in good_dev + list(FstabData.virtual_dev) + bad_dev :
        fstab += "%s%s %s %s   %s\t0                0    \n" \
            % (" " * random.randint(0, 10), dev, get_path(dev), "%s_type" % dev, "%s_opt" % dev)
    for dev in extra_good_dev :
        for a in [ "UUID=%s" % info[dev]["FS_UUID"], "/dev/disk/by-uuid/%s" % info[dev]["FS_UUID"] ]:
            fstab += "%s%s %s %s   %s\t0                0    \n" \
                % (" " * random.randint(0, 10), a, get_path(a), "%s_type" % a, "%s_opt" % a)
    for dev in extra_extra_good_dev :
        for a in [ "LABEL=%s" % info[dev]["FS_LABEL"], "/dev/disk/by-label/%s" % info[dev]["FS_LABEL"] ] :
            fstab += "%s%s %s %s   %s\t0                0    \n" \
                % (" " * random.randint(0, 10), a, get_path(a), "%s_type" % a, "%s_opt" % a)
    fstab += "    #0    1 2         3\t4   5        6      7\n"
    fstab += "\t#0    1 2         3\t4   5       \n"
    fstab += " # 0    1 2         3\t4   5    \t   \n"
    fstab += good_comment
    fstab += "         # 0    1 2         3\t4   5     6  \n"
    fstab += "   \t # 0    1 2         3\t4   5     6  \n"
    tmpfile = os.tmpfile()
    tmpfile.write(fstab)
    tmpfile.seek(0)
    mntfile = open(TESTFSTAB, "w")
    shutil.copyfileobj(tmpfile, mntfile)
    mntfile.close()
    tmpfile.close()
    

class AInitMntFile(unittest.TestCase) :

    create_fstab()
    fstab = MntFile(TESTFSTAB)
    
    def testAcceptOnlyNotIgnoredDevice(self) :
        ''' only keep not ignored device '''

        lenght = len(good_dev) + len(extra_good_dev) * 2 + len(extra_extra_good_dev) * 2
        self.assertEqual(len(self.fstab), lenght)
        for device in [ k for k in good_dev if k not in extra_good_dev ] :
            self.assertEqual(len(self.fstab.search(device, keys=["DEVICE"])), 1)
        for device in [ k for k in extra_good_dev if not k in extra_extra_good_dev ] :
            self.assertEqual(len(self.fstab.search(device, keys=["DEVICE"])), 3)
        for device in extra_extra_good_dev :
            self.assertEqual(len(self.fstab.search(device, keys=["DEVICE"])), 5)
        
    def testCheckPathTypeOption(self) :
        ''' check that path, type and option are pass correctly '''
    
        for device in good_dev :
            self.assertEqual(self.fstab[get_path(device)]["FSTAB_PATH"], get_path(device))
            self.assertEqual(self.fstab[get_path(device)]["FSTAB_TYPE"], "%s_type" % device)
            self.assertEqual(self.fstab[get_path(device)]["FSTAB_OPTION"], "%s_opt" % device)
        for device in extra_good_dev :
            for dev in self.fstab.search(device) :
                self.assertEqual(self.fstab[get_path(dev)]["FSTAB_PATH"], get_path(self.fstab[dev]["FSTAB_NAME"]))
                self.assertEqual(self.fstab[get_path(dev)]["FSTAB_TYPE"], "%s_type" % self.fstab[dev]["FSTAB_NAME"])
                self.assertEqual(self.fstab[get_path(dev)]["FSTAB_OPTION"], "%s_opt" % self.fstab[dev]["FSTAB_NAME"])
        for device in extra_extra_good_dev :
            for dev in self.fstab.search(device) :
                self.assertEqual(self.fstab[get_path(dev)]["FSTAB_PATH"], getpath(self.fstab[dev]["FSTAB_NAME"]))
                self.assertEqual(self.fstab[get_path(dev)]["FSTAB_TYPE"], "%s_type" % self.fstab[dev]["FSTAB_NAME"])
                self.assertEqual(self.fstab[get_path(dev)]["FSTAB_OPTION"], "%s_opt" % self.fstab[dev]["FSTAB_NAME"])       

    def testStoreBadDeviceAsOther(self) :
        ''' check that bad device are stored in other '''
     
        self.assertEqual(len(self.fstab.other), len(bad_dev))
        for i in range(len(self.fstab.other)) :
            self.assertEqual(self.fstab.other[i]["FSTAB_NAME"], bad_dev[i])

    def testStoreVirtDeviceAsDev(self) :
        ''' check that virtual device are stored in dev '''
    
        self.assertEqual(len(self.fstab.dev), len(FstabData.virtual_dev))
        for i in range(len(self.fstab.dev)) :
            self.assertEqual(self.fstab.dev[i]["FSTAB_NAME"], FstabData.virtual_dev[i])
        
    def testKeepOnlyGoodComment(self) :
        ''' check that only good comment are keep'''
        
        self.assertEqual(len(self.fstab.comment), 1)
        self.assertEqual(self.fstab.comment[0]["FSTAB_NAME"], good_comment.strip().split()[0])
        
class BSanityCheck(unittest.TestCase) :
    
    def testApplySanity(self) :
        ''' Check that we keep the same data after 20 apply '''
    
        create_fstab()
        fstab = MntFile(TESTFSTAB)
        fstab.apply()
        fstab = MntFile(TESTFSTAB)
        for i in range(20) :
            fstab.apply()
        self.assertEqual(fstab, MntFile(TESTFSTAB))
        
    def testCopySanity(self) :
        ''' Check that we keep the same data after 20 copy '''
    
        create_fstab()
        fstab = MntFile(TESTFSTAB)
        fstab2 = fstab.copy()
        for i in range(20) :
            fstab2 = fstab2.copy()
        self.assertEqual(fstab, fstab2)
        
class CFunctionnalityTest(unittest.TestCase):
        
    def testAddRemove(self) :
        ''' Check Add/Remove functionnality '''
        
        create_fstab()
        fstab = MntFile(TESTFSTAB)
        for dev in good_dev :
            for i in [str(k) for k in range(5) ] :
                fstab.add([dev, get_path(dev) + "_%s" % i])
        fstab.apply()
        fstab = MntFile(TESTFSTAB)
        for dev in good_dev :
            l = 5 + 1 + 2 * int(dev in extra_good_dev) + 2 * int(dev in extra_extra_good_dev)
            self.assertEqual(len(fstab.search(dev, keys=["DEVICE"])), l)
            for i in [str(k) for k in range(5) ] :
                self.assertEqual(len(fstab.search(get_path(dev) + "_%s" % i)), 1)
                entry = fstab[get_path(dev) + "_%s" % i]
                self.assertEqual(entry["FSTAB_OPTION"], entry.defaultopt())
                fstab.remove(entry)
        fstab.apply()
        fstab = MntFile(TESTFSTAB)
        for dev in good_dev :
            l = 1 + 2 * int(dev in extra_good_dev) + 2 * int(dev in extra_extra_good_dev)
            self.assertEqual(len(fstab.search(dev, keys=["DEVICE"])), l)

if __name__ == "__main__" :
    
    unittest.main()
    
    
